@@ -462,9 +462,21 @@ int initIB(ArgStruct *p)
      * QP type.  (RC=reliable connection, UC=unreliable.. etc.) defined 
      *   in the verbs header.
      */
+
   memset(&qp_init_attr, 0, sizeof(struct ibv_qp_init_attr)); 
-  qp_init_attr.cap.max_recv_wr    = max_wq; /* Max outstanding WR on RQ      */
-  qp_init_attr.cap.max_send_wr    = max_wq; /* Max outstanding WR on SQ      */
+/*
+ * Set the Max outstanding QP WRs, can be set via max_wr global
+ * or if this is greater than the max reported by the HCA, set
+ * to max reported value.
+ */
+if (hca_attr.max_qp_wr < max_wq ) {
+  LOGPRINTF(("Reduced max_wq to hca reported max of %d", hca_attr.max_qp_wr));
+  qp_init_attr.cap.max_send_wr = hca_attr.max_qp_wr;
+  qp_init_attr.cap.max_recv_wr = hca_attr.max_qp_wr;
+} else {
+  qp_init_attr.cap.max_recv_wr    = max_wq;
+  qp_init_attr.cap.max_send_wr    = max_wq;
+}
   qp_init_attr.cap.max_recv_sge   = 1; /* Max scatter/gather entries on RQ */
   qp_init_attr.cap.max_send_sge   = 1; /* Max scatter/gather entries on SQ */
   qp_init_attr.recv_cq            = r_cq_hndl; /* CQ handle for RQ         */
