@@ -18,6 +18,9 @@
 /*     * pvm.h           ---- Include file for PVM calls and data structs    */
 /*     * tcgmsg.c        ---- TCGMSG calls source                            */
 /*     * tcgmsg.h        ---- Include file for TCGMSG calls and data structs */
+/*     * udapl.c         ---- uDAPL calls source                             */
+/*     * knem.c          ---- knem calls source                              */
+/*     * vmsplice.c      ---- vmsplice calls source                          */
 /*****************************************************************************/
 
 #include "netpipe.h"
@@ -97,7 +100,7 @@ int main(int argc, char **argv)
 #if ! defined(TCGMSG)
 
     /* Parse the arguments. See Usage for description */
-    while ((c = getopt(argc, argv, "AXSO:rIiPszgfaB2h:p:o:l:u:b:m:n:t:c:d:D:P:")) != -1)
+    while ((c = getopt(argc, argv, "AXSO:rIiszgfaB2h:p:o:l:u:b:m:n:t:c:d:D:P:")) != -1)
     {
         switch(c)
         {
@@ -148,7 +151,7 @@ int main(int argc, char **argv)
 
             case 's': streamopt = 1;
                       printf("Streaming in one direction only.\n\n");
-#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB)
+#if defined(TCP) && !defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT) 
                       printf("Sockets are reset between trials to avoid\n");
                       printf("degradation from a collapsing window size.\n\n");
 #endif
@@ -174,7 +177,7 @@ int main(int argc, char **argv)
             case 'u': end = atoi(optarg);
                       break;
 
-#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB)
+#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
             case 'b': /* -b # resets the buffer size, -b 0 keeps system defs */
                       args.prot.sndbufsz = args.prot.rcvbufsz = atoi(optarg);
                       break;
@@ -184,7 +187,7 @@ int main(int argc, char **argv)
                          /* end will be maxed at sndbufsz+rcvbufsz */
                       printf("Passing data in both directions simultaneously.\n");
                       printf("Output is for the combined bandwidth.\n");
-#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB)
+#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
                       printf("The socket buffer size limits the maximum test size.\n\n");
 #endif
                       if( streamopt ) {
@@ -258,7 +261,7 @@ int main(int argc, char **argv)
                       break;
 #endif /* MPI2 */
 
-#if defined(INFINIBAND)
+#if defined(INFINIBAND) && !defined(DAT)
             case 'm': switch(atoi(optarg)) {
                         case 256: args.prot.ib_mtu = MTU256;
                           break;
@@ -278,7 +281,7 @@ int main(int argc, char **argv)
                       break;
 #endif
 
-#if defined(OPENIB)
+#if defined(OPENIB) && !defined(DAT)
             case 'm': switch(atoi(optarg)) {
                         case 256: args.prot.ib_mtu = IBV_MTU_256;
                           break;
@@ -303,7 +306,7 @@ int main(int argc, char **argv)
                       break;
 #endif
 
-#if defined(OPENIB) || defined(INFINIBAND)
+#if ( defined(OPENIB) || defined(INFINIBAND) ) && !defined(DAT)
             case 't': if( !strcmp(optarg, "send_recv") ) {
                          printf("Using Send/Receive communications\n");
                          args.prot.commtype = NP_COMM_SENDRECV;
@@ -346,6 +349,65 @@ int main(int argc, char **argv)
                       }
                       break;
 #endif
+
+#if defined(DAT)
+            case 't': if( !strcmp(optarg, "send_recv") ) {
+                         printf("Using Send/Receive communications\n");
+                         args.prot.commtype = NP_COMM_SENDRECV;
+                      } else if( !strcmp(optarg, "rdma_write") ) {
+                         printf("Using RDMA Write communications\n");
+                         args.prot.commtype = NP_COMM_RDMAWRITE;
+                      } else {
+                         fprintf(stderr, "Invalid transfer type "
+                                 "specified, please choose one of:\n\n"
+                                 "\tsend_recv\t\tUse Send/Receive communications\t(default)\n"
+                                 "\trdma_write\t\tUse RDMA Write communications\n");
+                         exit(-1);
+                      }
+                      break;
+
+            case 'c': if( !strcmp(optarg, "local_poll") ) {
+                         printf("Using local polling completion\n");
+                         args.prot.comptype = NP_COMP_LOCALPOLL;
+                      } else if( !strcmp(optarg, "dq_poll") ) {
+                         printf("Using EVD Dequeue to poll for completion\n");
+                         args.prot.comptype = NP_COMP_DQPOLL;
+                      } else if( !strcmp(optarg, "evd_wait") ) {
+                         printf("Using EVD Wait for completion\n");
+                         args.prot.comptype = NP_COMP_EVD;
+                      } else if( !strcmp(optarg, "cno_wait") ) {
+                         printf("Using CNO Wait for completion\n");
+                         args.prot.comptype = NP_COMP_CNO;
+                      } else {
+                         fprintf(stderr, "Invalid completion type specified, "
+                                 "please choose one of:\n\n"
+                                 "\tlocal_poll\tWait for last byte of data (default)\n"
+                                 "\tdq_poll\t\tUse EVD Dequeue to poll for completion\n"
+                                 "\tevd_wait\tUse EVD wait for completion events\n"
+                                 "\tcno_wait\tUse CNO wait for completion events\n\n");
+                         exit(-1);
+                      }
+                      break;
+#endif
+
+#if defined(KNEM)
+            case 't': if( !strcasecmp(optarg, "copy") ) {
+                         printf("Using software copy mode\n");
+                         args.prot.flags = 0;
+                      } else if( !strcasecmp(optarg, "dma") ) {
+                         printf("Using DMA copy mode\n");
+                         args.prot.flags = KNEM_FLAG_DMA;
+                      } else {
+                         fprintf(stderr, "Invalid transfer type "
+                                 "specified, please choose one of:\n\n"
+                                 "\tcopy\t\tUse software copy mode\t(default)\n"
+                                 "\tdma\t\tUse DMA copy mode\n");
+                         exit(-1);
+                      }
+		      break;
+
+#endif
+
 	    case 'P': 
 		      args.port = atoi(optarg);
 		      break;
@@ -374,9 +436,27 @@ int main(int argc, char **argv)
    	};
 #endif /* ! defined TCGMSG */
 
-#if defined(OPENIB) || defined(INFINIBAND)
+#if defined(OPENIB) || defined(INFINIBAND) || defined(DAT)
+
+#if defined(DAT)
+   // if using RDMA_WRITE, no need to prepost receives since no receive requests are 
+   // consumed by the RDMA_WRITE operations. however, if using a completion notification
+   // method other than LOCAL_POLL along with RDMA_WRITES, the handshake receive packet
+   // will need to be preposted.
+   if ( (NP_COMM_RDMAWRITE == args.prot.commtype) && (NP_COMP_LOCALPOLL == args.prot.comptype))
+   {
+       asyncReceive = 0;
+   } 
+   else
+   {
+       asyncReceive = 1;
+       fprintf(stderr, "Preposting asynchronous receive.\n");
+   }
+#else
    asyncReceive = 1;
    fprintf(stderr, "Preposting asynchronous receives (required for Infiniband)\n");
+#endif
+
    if(args.bidir && (
           (args.cache && args.prot.commtype == NP_COMM_RDMAWRITE) || /* rdma_write only works with no-cache mode */
           (!args.preburst && args.prot.commtype != NP_COMM_RDMAWRITE) || /* anything besides rdma_write requires prepost burst */
@@ -419,7 +499,7 @@ int main(int argc, char **argv)
       end = args.upper;
       if( args.tr ) {
          printf("The upper limit is being set to %d Bytes\n", end);
-#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB)
+#if defined(TCP) && ! defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
          printf("due to socket buffer size limitations\n\n");
 #endif
    }  }
@@ -580,8 +660,8 @@ int main(int argc, char **argv)
 /*               } else if (len == start) {*/
 /*                   nrepeat = MAX( RUNTM/( 0.000020 + start/(8*1000) ), TRIALS);*/
                } else {
-                   nrepeat = MAX((RUNTM / ((double)args.bufflen /
-                                  (args.bufflen - inc + 1.0) * tlast)),TRIALS);
+                   nrepeat = MAX((RUNTM / (((double)args.bufflen /
+                                  (args.bufflen - inc + 1.0)) * tlast)),TRIALS);
                }
                SendRepeat(&args, nrepeat);
            }
@@ -1032,11 +1112,11 @@ void VerifyIntegrity(ArgStruct *p)
 void PrintUsage()
 {
     printf("\n NETPIPE USAGE \n\n");
-#if ! defined(INFINIBAND) && !defined(OPENIB)
+#if ! defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
     printf("a: asynchronous receive (a.k.a. preposted receive)\n");
 #endif
     printf("B: burst all preposts before measuring performance\n");
-#if (defined(TCP) || defined(TCP6)) && ! defined(INFINIBAND)
+#if (defined(TCP) || defined(TCP6)) && ! defined(INFINIBAND) && !defined(DAT)
     printf("b: specify TCP send/receive socket buffer sizes\n");
 #endif
 
@@ -1045,6 +1125,12 @@ void PrintUsage()
            "   valid types: local_poll, vapi_poll, event\n"
            "   default: local_poll\n");
 #endif
+
+#if defined(DAT)
+    printf("c: specify type of completion <-c type>\n"
+           "   valid types: local_poll, dq_poll, evd_wait, cno_wait\n"
+           "   default: local_poll\n");
+#endif
     
 #if defined(MPI2)
     printf("g: use get instead of put\n");
@@ -1052,7 +1138,7 @@ void PrintUsage()
     printf("   all MPI-2 implementations\n");
 #endif
 
-#if defined(TCP) || defined(TCP6) || defined(SCTP) || defined(SCTP6) || defined(INFINIBAND) || defined(OPENIB)
+#if defined(TCP) || defined(TCP6) || defined(SCTP) || defined(SCTP6) || defined(INFINIBAND) || defined(OPENIB) || defined(DAT)
     printf("h: specify hostname of the receiver <-h host>\n");
 #endif
 
@@ -1072,7 +1158,7 @@ void PrintUsage()
     printf("p: set the perturbation number <-p 1>\n"
            "   (default = 3 Bytes, set to 0 for no perturbations)\n");
 
-#if (defined(TCP) || defined(TCP6) || defined(SCTP) || defined(SCTP6)) && ! defined(INFINIBAND) && !defined(OPENIB)
+#if (defined(TCP) || defined(TCP6) || defined(SCTP) || defined(SCTP6)) && ! defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
     printf("r: reset sockets for every trial\n");
 #endif
 
@@ -1085,7 +1171,7 @@ void PrintUsage()
     printf("t: specify type of communications <-t type>\n"
            "   valid types: send_recv, send_recv_with_imm,\n"
            "                rdma_write, rdma_write_with_imm\n"
-           "   defaul: send_recv\n");
+           "   default: send_recv\n");
 #endif
 #if defined(OPENIB)
     printf("D: specify an OpenFabrics device/port combination\n"
@@ -1098,7 +1184,26 @@ void PrintUsage()
            "   No specification will result in using the first\n"
            "   active port on any valid device.\n");
 #endif
+
+#if defined(DAT)
+    printf("t: specify type of communications <-t type>\n"
+           "   valid types: send_recv, rdma_write\n"
+           "   default: send_recv\n");
+#endif
     
+#if defined(KNEM)
+    printf("t: specify type of communications <-t type>\n"
+           "   valid types: copy, dma\n"
+           "   default: copy\n");
+#endif
+
+#if defined(KNEM) || defined(VMSPLICE)
+    printf("h: specify a filename to use to synchronize\n"
+           "   When run with this option, two copies of Netpipe\n"
+           "   must be invoked on the same machine with the\n"
+           "   -h argument values.\n");
+#endif
+
     printf("u: upper bound stop value e.g. <-u 1048576>\n");
  
 #if defined(MPI)
@@ -1110,7 +1215,7 @@ void PrintUsage()
 #if defined(MPI)
     printf("   May need to use -a to choose asynchronous communications for MPI/n");
 #endif
-#if (defined(TCP) || defined(TCP6) || defined(SCTP) || defined (SCTP6)) && !defined(INFINIBAND) && !defined(OPENIB)
+#if (defined(TCP) || defined(TCP6) || defined(SCTP) || defined (SCTP6)) && !defined(INFINIBAND) && !defined(OPENIB) && !defined(DAT)
     printf("   The maximum test size is limited by the TCP buffer size\n");
 #endif
 #if defined(TCP)
@@ -1188,7 +1293,7 @@ void InitBufferData(ArgStruct *p, int nbytes, int soffset, int roffset)
 
     memset(p->s_buff, 'b', nbytes+soffset);
 }
-#if !defined(OPENIB) && !defined(INFINIBAND) && !defined(ARMCI) && !defined(LAPI) && !defined(GPSHMEM) && !defined(SHMEM) && !defined(GM) 
+#if !defined(OPENIB) && !defined(INFINIBAND) && !defined(DAT) && !defined(ARMCI) && !defined(LAPI) && !defined(GPSHMEM) && !defined(SHMEM) && !defined(GM) 
 
 void MyMalloc(ArgStruct *p, int bufflen, int soffset, int roffset)
 {
