@@ -23,10 +23,19 @@
 #include <stdlib.h>         /* malloc(3) */
 #include <unistd.h>         /* getopt, read, write, ... */
 
+/* Handle the case of building on MacOS X */
+#if defined(__APPLE__)
+#include <stdint.h>
+#endif 
+
 #ifdef INFINIBAND
 #include <ib_defs.h> /* ib_mtu_t */
 #endif
 
+#ifdef OPENIB
+#include <infiniband/verbs.h>	/* enum ibv_mtu */
+#endif
+
 #ifdef FINAL
   #define  TRIALS             7
   #define  RUNTM              0.25
@@ -72,10 +81,20 @@
       IB_mtu_t                ib_mtu;   /* MTU Size for Infiniband HCA    */
       int                     commtype; /* Communications type            */
       int                     comptype; /* Completion type                */
+#endif
+#if defined(OPENIB)
+      enum ibv_mtu            ib_mtu;   /* MTU Size for Infiniband HCA    */
+      int                     commtype; /* Communications type            */
+      int                     comptype; /* Completion type                */
+      char                    *device_and_port; /* Local port specification */
+#endif
+#if defined(DAT)
+      int                     commtype; /* Communications type            */
+      int                     comptype; /* Completion type                */
 #endif
   };
 
-#if defined(INFINIBAND)
+#if defined(INFINIBAND) || defined(OPENIB)
 enum completion_types {
    NP_COMP_LOCALPOLL,  /* Poll locally on last byte of data     */
    NP_COMP_VAPIPOLL,   /* Poll using vapi function              */
@@ -89,6 +108,87 @@ enum communication_types {
 };
 #endif
 
+#if defined(DAT)
+enum completion_types {
+   NP_COMP_LOCALPOLL,  /* Poll locally on last byte of data */
+   NP_COMP_DQPOLL,     /* Poll using uDAPL dequeue function */
+   NP_COMP_EVD,        /* Don't poll, use uDAPL EVD wait    */
+   NP_COMP_CNO         /* Don't poll, use uDAPL CNO wait    */
+};
+enum communication_types {
+   NP_COMM_SENDRECV,           /* Communication with send/receive            */
+   NP_COMM_RDMAWRITE,          /* Communication with rdma write              */
+};
+#endif
+
+#elif defined(SCTP6)
+  #include <netdb.h>
+  #include <sys/socket.h>
+  #include <netinet/in.h>
+  #include <netinet/sctp.h>
+  #include <arpa/inet.h>
+  
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct
+  {
+      struct sockaddr_in6     sin1;   /* socket structure #1              */
+      struct sockaddr_in6     sin2;   /* socket structure #2              */
+      int                     nodelay;  /* Flag for TCP nodelay           */
+      struct hostent          *addr;    /* Address of host                */
+      int                     sndbufsz; /* Size of TCP send buffer        */
+      int                     rcvbufsz; /* Size of TCP receive buffer     */
+  };
+
+#elif defined(SCTP)
+  #include <netdb.h>
+  #include <sys/socket.h>
+  #include <netinet/in.h>
+  #include <netinet/sctp.h>
+  #include <arpa/inet.h>
+  
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct
+  {
+      struct sockaddr_in      sin1,   /* socket structure #1              */
+                              sin2;   /* socket structure #2              */
+      int                     nodelay;  /* Flag for TCP nodelay           */
+      struct hostent          *addr;    /* Address of host                */
+      int                     sndbufsz, /* Size of TCP send buffer        */
+                              rcvbufsz; /* Size of TCP receive buffer     */
+  };
+
+#elif defined(TCP6)
+  #include <netdb.h>
+  #include <sys/socket.h>
+  #include <netinet/in.h>
+  #include <netinet/tcp.h>
+  #include <arpa/inet.h>
+  
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct
+  {
+      struct sockaddr_in6     sin1;   /* socket structure #1              */
+      struct sockaddr_in6     sin2;   /* socket structure #2              */
+      int                     nodelay;  /* Flag for TCP nodelay           */
+      struct hostent          *addr;    /* Address of host                */
+      int                     sndbufsz; /* Size of TCP send buffer        */
+      int                     rcvbufsz; /* Size of TCP receive buffer     */
+  };
+
+#elif defined(IPX)
+  #include <netdb.h>
+  #include <sys/socket.h>
+  #include <netipx/ipx.h>
+  
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct
+  {
+      struct sockaddr_ipx     sipx1;   /* socket structure #1              */
+      struct sockaddr_ipx     sipx2;   /* socket structure #2              */
+      int                     sndbufsz; /* Size of send buffer        */
+      int                     rcvbufsz; /* Size of receive buffer     */
+  };
+
 #elif defined(MPI)
   typedef struct protocolstruct ProtocolStruct;
   struct protocolstruct 
@@ -160,6 +260,16 @@ enum communication_types {
   };
 
 
+#elif defined(MX)
+  #include "myriexpress.h"
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct
+  {
+      mx_endpoint_t ep;
+      mx_endpoint_addr_t addr;
+      mx_request_t rreq;
+  };
+
 #elif defined(GM)
   #include "gm.h"
   typedef struct protocolstruct ProtocolStruct;
@@ -184,10 +294,19 @@ enum communication_types {
               id_nbor;       /* My neighbor's port id */
   }
 
-#elif defined(MEMCPY)
+#elif defined(MEMCPY) || defined(VMSPLICE)
   typedef struct protocolstruct ProtocolStruct;
   struct protocolstruct { int nothing; };
 
+#elif defined(KNEM)
+#include "knem_io.h"
+
+  typedef struct protocolstruct ProtocolStruct;
+  struct protocolstruct { 
+      int flags; 
+      char *shared_filename;
+  };
+
 #elif defined(DISK)
   typedef struct protocolstruct ProtocolStruct;
   struct protocolstruct {
@@ -197,7 +316,7 @@ enum communication_types {
   };
 
 #else
-  #error "One of TCP, MPI, PVM, TCGMSG, LAPI, SHMEM, ATOLL, MEMCPY, DISK must be defined during compilation"
+  #error "One of TCP, TCP6, SCTP, SCTP6, IPX, MPI, PVM, TCGMSG, LAPI, SHMEM, MX, GM, ATOLL, MEMCPY, DISK, VMSPLICE, or KNEM must be defined during compilation"
 
 #endif
 
@@ -234,7 +353,7 @@ struct argstruct
     int      reset_conn;    /* Reset connection flag                         */
     int      soffset,roffset;
     int      syncflag; /* flag for using sync sends vs. normal sends in MPI mod*/
-
+    int	     use_sdp;       /* Use AF_INET_SDP instead of AF_INET */
     /* Now we work with a union of information for protocol dependent stuff  */
     ProtocolStruct prot;
 };
